@@ -16,7 +16,7 @@ use strict;
 use warnings;
 no warnings 'once';
 
-our $VERSION = '0.009001';
+our $VERSION = '0.009002';
 $VERSION = eval $VERSION;
 
 use Carp ();
@@ -309,7 +309,7 @@ sub CLONE {
 sub _update_ex_refs {
   for my $id ( keys %EXCEPTIONS ) {
     next
-      if $EXCEPTIONS{$id};
+      if defined $EXCEPTIONS{$id};
     delete $EXCEPTIONS{$id};
     delete $PACKAGES{$id};
     delete $MESSAGES{$id};
@@ -327,7 +327,7 @@ sub _convert {
       if ! do {no strict 'refs'; defined &{"Devel::Confess::_Attached::DESTROY"} };
     my $message;
     my $id = refaddr($ex);
-    if ($EXCEPTIONS{$id}) {
+    if (defined $EXCEPTIONS{$id}) {
       return @_
         if $ex->isa("Devel::Confess::_Attached");
 
@@ -396,16 +396,19 @@ sub _convert {
 
   my $source_trace;
   $out =~ s/^(={75}\ncontext for .*^={75}\n\z)//ms
-    and $source_trace = $1;
+    and $source_trace = $1
+    if $OPTIONS{source} || $OPTIONS{evalsource};
   my $trace = _stack_trace();
   $trace =~ s/^(.*\n?)//;
   my $where = $1;
   my $new_source_trace;
   $trace =~ s/^(={75}\ncontext for .*^={75}\n\z)//ms
-    and $new_source_trace = $1;
+    and $new_source_trace = $1
+    if $OPTIONS{source} || $OPTIONS{evalsource};
   my $find = $where;
   $find =~ s/(\.?\n?)\z//;
-  $out =~ s/(\Q$find\E(?: during global destruction)?(\.?\n?))(?:\Q$trace\E)?\z//
+  my $trace_re = length $trace ? "(?:\Q$trace\E)?" : '';
+  $out =~ s/(\Q$find\E(?: during global destruction)?(\.?\n?))$trace_re\z//
     and $where = $1;
   if (defined $source_trace) {
     if (defined $new_source_trace) {
