@@ -35,17 +35,22 @@ use Scalar::Util qw(blessed refaddr reftype);
       $level++;
     }
     local $CarpLevel = $CarpLevel + $level;
+    no strict 'refs';
+    local *{"threads::tid"} = \&threads::tid
+      if defined &threads::tid && !defined &{"threads::tid"};
     &longmess;
   };
-} : Carp->VERSION <= 1.04 ? eval q{
+} : eval q{
   package
     Carp;
-  our ($CarpLevel);
   sub {
     local $INC{'Carp/Heavy.pm'} = $INC{'Carp/Heavy.pm'} || 1;
+    no strict 'refs';
+    local *{"threads::tid"} = \&threads::tid
+      if defined &threads::tid && !defined &{"threads::tid"};
     &longmess;
   };
-} : \&Carp::longmess;
+} or die $@;
 
 if (defined &Carp::format_arg && $Carp::VERSION < 1.32) {
   my $format_arg = \&Carp::format_arg;
@@ -53,7 +58,7 @@ if (defined &Carp::format_arg && $Carp::VERSION < 1.32) {
     package
       Carp;
     our $in_recurse;
-    $format_arg; # capture
+    $format_arg if 0; # capture
     no warnings 'redefine';
     sub format_arg {
       if (! $in_recurse) {
@@ -63,7 +68,7 @@ if (defined &Carp::format_arg && $Carp::VERSION < 1.32) {
 
         my $arg;
         if (
-          Scalar::Util::blessed($_[0])
+          Devel::Confess::_Util::blessed($_[0])
           && eval { $_[0]->can('CARP_TRACE') }
         ) {
           return $_[0]->CARP_TRACE;
@@ -78,21 +83,24 @@ if (defined &Carp::format_arg && $Carp::VERSION < 1.32) {
       }
       $format_arg->(@_);
     }
+    1;
   } or die $@;
 }
 
-*_str_val = eval q{
-  sub {
+eval q{
+  sub _str_val {
     no overloading;
     "$_[0]";
-  };
-} || eval q{
-  sub {
+  }
+  1;
+} or eval q{
+  sub _str_val {
     my $class = &blessed;
     return "$_[0]" unless defined $class;
     return sprintf("%s=%s(0x%x)", $class, &reftype, &refaddr);
-  };
-};
+  }
+  1;
+} or die $@;
 
 {
   if (defined ${^GLOBAL_PHASE}) {
